function [ xax, out ] = theoDataCollector( m, n, cRange, muRange, li, Q, delta, xAxisType , theoFn, origTitle )
%THEODATACOLLECTOR computes one bound and outputs the results.  This is an
%internal function only intended to be used by the kARiPlotter package.
%
%   Inputs:
%
%   m: The number of desired rows*
%
%   n: The number of desired columns*
%
%   cRange: The c values to be tested
%     If xAxisType==1, then cRange should be a scalar.  If xAxisType==2,
%     then cRange should be a vector.
%
%   muRange: The mu values to be tested*
%     If xAxisType==1, then muRange should be a vector.  If xAxisType==2,
%     then muRange should be a scalar.
%
%   li: If xAxisType==2, then these are the leverage scores to be tested.
%     If xAxisType==1, then this will be a function that returns the
%     desired leverage scores.
%
%   mtxM: Either a mXn matrix or a function*
%     mtxM defines the matrix that we are going to test.  If mtxM is a mXn
%     matrix, then we will use mtxM for our experiments.  If mtxM is a
%     function, then mtxM(m,n,mu) will return the matrix that we will use
%     for our experiments.
%
%   delta: The desired failure rate of our bound*
%
%   xAxisType: A scalar that should be either 1 or 2
%     If xAxisType==1, then we are testing multiple mu values and mu will
%     be on the x-axis of our final plot.  If xAxisType==2, then we are
%     testing multiple c values and c will be on the x-axis of our final
%     plot.
%
%   sampleM: A function that defines how we will sample rows from the
%   matrix
%     This function will be passed to the runSampling function.
%
%   origTitle: A string defining the first part of the title for the
%   waitbar.
%
%   Outputs:
%
%   xax: A vector of the x-values for the desired plot
%
%   yax: A vector of the y-values for the desired plot
%
%
%  * If needed.  If this variable isn't needed, then we will pass an empty
%  vector.




clear(func2str(theoFn))%remove any persistent variables (just in case!)

%---Code for waitbar---
%we update the waitbar every .01% done, or every 10 seconds, whichever
%comes first
pdone=0;
lastTime=0;
wtBr=waitbar(0,origTitle);
%the current title of the waitbar
%----------------------

skp=0;%this is used to delete the data points where a theoretical bound could not be established.


if xAxisType==1
    computeQYN=false;
    computeLiYN=false;
    Qtemp=NaN;
    liTemp=NaN;
    theoFnInfo=theoFn('info');
    if isnumeric(Q) && ~isempty(Q)
        error('You cannot test multiple mu with one matrix')
    elseif isa(Q,'function_handle') %if Q is a function handle
        if theoFnInfo.inputsNeeded.Q  %only generate Q if the theoFn needs it.
            computeQYN=true;
            computeLiYN=true;
        end
    elseif isnumeric(Q) && isempty(Q)
        computeQYN=false;
    else
        error('Unknown Q data type')
    end
    if isnumeric(li) && ~isempty(li)
        error('You cannot test multiple mu with one leverage score distribution.')
    elseif isa(li,'function_handle') %if li is a function handle
        if theoFnInfo.inputsNeeded.li
            computeLiYN=true;
        end
    elseif isnumeric(li) && isempty(li)
        if computeLiYN
            error('You cannot test multiple matrix with one leverage score distribution.')
        end
    else
        error('Unknown li data type')
    end
    
    
    out=zeros(size(muRange));%run data will be stored here
    xax=muRange;%xaxis data will be stored here
    
    
    tic
    for i=1:length(muRange)
        if computeLiYN
            liTemp = li(m,n,muRange(i));
            clear(func2str(theoFn))%remove any persistent variables (just in case they depend on li!)
        end
        if computeQYN
            Qtemp=Q(m,n,liTemp, muRange(i)); %generate the matrix (Q), and compute
            clear(func2str(theoFn))%remove any persistent variables (just in case they depend on Q!)
        end
        
        fnOut=theoFn(m,n,cRange, muRange(i), delta, liTemp, Qtemp);
        
        if ischar(fnOut) %theoFn will return 'badEps' if a bound could not be computed for these values
            skp=skp+1;
        else
            out(i-skp)=fnOut;
            xax(i-skp)=muRange(i);
        end
        
        
        %---Code for waitbar---
        if pdone<i/length(muRange) || toc>lastTime+10
            lastTime=toc;
            pdone=pdone+.01;
            ttt=toc/i*(length(muRange)-i);
            if ttt>3600
                tLeft=[num2str(ttt/3600),' hours'];
            elseif ttt>60
                tLeft=[num2str(ttt/60),' minutes'];
            else
                tLeft=[num2str(ttt),' seconds'];
            end
            waitbar(i/length(muRange),wtBr,[origTitle,'| ETR: ',tLeft])
        end
        %----------------------
        
    end
elseif xAxisType==2
    
    if ~isnumeric(Q)
        theoFnInfo=theoFn('info');
        if theoFnInfo.needQ  %only generate Q if the theoFn needs it.
            [Q,~]=Q(m,n,muRange);%remember, Q is a function if we are here
        else
            Q=NaN;
        end
    end
    
    out=zeros(size(cRange));%run data will be stored here
    xax=cRange;%xaxis data will be stored here
    
    tic
    for i=1:length(cRange)
        
        fnOut=theoFn(m,n,cRange(i), muRange, delta, li,Q);
        
        if ischar(fnOut)
            skp=skp+1;
        else
            out(i-skp)=fnOut;
            xax(i-skp)=cRange(i);
        end
        
        %---Code for waitbar---
        if pdone<i/length(cRange) || toc>lastTime+10
            lastTime=toc;
            pdone=pdone+.01;
            ttt=toc/i*(length(cRange)-i);
            if ttt>3600
                tLeft=[num2str(ttt/3600),' hours'];
            elseif ttt>60
                tLeft=[num2str(ttt/60),' minutes'];
            else
                tLeft=[num2str(ttt),' seconds'];
            end
            waitbar(i/length(cRange),wtBr,[origTitle,'| ETR: ',tLeft])
        end
        %----------------------
    end
    
else
    error('invalid xAxisType')
end
xax=xax(1:end-skp);%remove unused entries
out=out(1:end-skp);
%---Code for waitbar---
delete(wtBr)
%----------------------
clear(func2str(theoFn))%remove any persistent variables (just in case!)
end

